#include "booster_lite.h"


/******************************************************
*******************************************************
*
* Globals
*
*******************************************************
******************************************************/
cvar_t * cvar_booster_lite_mode;
cvar_t * cvar_booster_lite_extra_sleep_frequency;


/******************************************************
*******************************************************
*
* Statics
*
*******************************************************
******************************************************/
static WSAEVENT *    SocketEvents = 0;
static SOCKET   *    Sockets      = 0;
static unsigned long list_next    = 0;
static unsigned long list_max     = 0;

static int ExtraSleepCounter = 0;


/******************************************************
*******************************************************
*
* Functions
*
*******************************************************
******************************************************/
//
void LogError(const char * format, ...)
{
	static char string[MAX_LOGMSG_LEN];
	
	va_list valist;
	
	va_start(valist, format);
	_vsnprintf(string, sizeof(string), format, valist);
	string[sizeof(string)-2] = 0;
	va_end(valist);
	
	LOG_ERROR(PLID, "%s", string);
}

//list clean up
void free_list(void)
{
	for(unsigned long i = 0; i < list_next; i++)
	{
		WSAEventSelect(Sockets[i], SocketEvents[i], 0);
		WSACloseEvent(SocketEvents[i]);
	}
	
	if(SocketEvents) 
		free(SocketEvents);
	if(Sockets) 
		free(Sockets);
	
	SocketEvents = 0;
	Sockets = 0;
	list_next = 0;
	list_max = 0;
}

//Changes import function for hooking and stores original pointer
unsigned long __fastcall ExchangeImport(HMODULE module, unsigned long funcAddress, unsigned long newAddress, unsigned long * oldAddress)
{
	union { 
		unsigned long mem;
		IMAGE_DOS_HEADER        * dos;
		IMAGE_NT_HEADERS        * pe;
		IMAGE_IMPORT_DESCRIPTOR * import;
	} mem;
	union {
		unsigned long mem;
		IMAGE_THUNK_DATA *thunk;
	} mem2;
	
	if(!module || !newAddress || !funcAddress)
		return(0);
	
	//Check if valid dos header
	mem.mem = (unsigned long)module;
	if(IsBadReadPtr(mem.dos, sizeof(*mem.dos)) || mem.dos->e_magic != IMAGE_DOS_SIGNATURE)
		return(0);
	
	//Get and check pe header
	mem.mem = (unsigned long)module + (unsigned long)mem.dos->e_lfanew;
	if(IsBadReadPtr(mem.pe, sizeof(*mem.pe)) || mem.pe->Signature != IMAGE_NT_SIGNATURE)
		return(0);
	
	//Get imports
	mem.mem = (unsigned long)module + (unsigned long)mem.pe->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress;
	if(mem.mem == (unsigned long)module)
		return(0);
	
	while(!IsBadReadPtr(mem.import, sizeof(*mem.import)) && mem.import->Name)
	{
		mem2.mem = (unsigned long)module + (unsigned long)mem.import->FirstThunk;
		while(!IsBadReadPtr(mem2.thunk, sizeof(*mem2.thunk)) && mem2.thunk->u1.Function)
		{
			if((unsigned long)mem2.thunk->u1.Function == (unsigned long)funcAddress)
			{
				if(oldAddress)
					*oldAddress = (unsigned long)mem2.thunk->u1.Function;
				
				VirtualProtect(&mem2.thunk->u1.Function, 4, PAGE_READWRITE, &mem.mem);
				if(!IsBadWritePtr(&mem2.thunk->u1.Function, sizeof(unsigned long)))
					*(unsigned long*)&mem2.thunk->u1.Function = (unsigned long)newAddress;
				VirtualProtect(&mem2.thunk->u1.Function, 4 , mem.mem, &mem.mem);
				
				return(1);
			}
				
			mem2.thunk++;
		}
		
		mem.import++;
	}
	
	return(0);
}

//Get (and fix) extra sleep frequency
static int GetExtraSleepValue(void)
{
	float ExtraSleepValue = floor(cvar_booster_lite_extra_sleep_frequency->value);
	
	if(ExtraSleepValue <= 0.0f)
	{
		ExtraSleepCounter = 0;
		return(1024);
	}
	else
		return((int)ExtraSleepValue);
}

//stops sleeping if new packet has been received.. and handles extra sleep feature.
static int __fastcall SimpleNetSleep(const int ExtraSleepValue, const DWORD dwSleep)
{
	//Check events and sleep
	DWORD Index = WSAWaitForMultipleEvents(list_next, SocketEvents, FALSE, dwSleep, FALSE);
	
	if(WSA_WAIT_FAILED == Index)
	{
		LogError("WSAWaitForMultipleEvents(0x%x,0x%x,0x%x,0x%x,0x%x) == WSA_WAIT_FAILED, error code: %d", list_next, SocketEvents, FALSE, dwSleep, FALSE, WSAGetLastError());
	}
	else if(WSA_WAIT_EVENT_0 >= Index && (WSA_WAIT_EVENT_0 + list_next - 1) <= Index)
	{
		//Event triggered.. break
		if(!WSAResetEvent(SocketEvents[Index - WSA_WAIT_EVENT_0]))
		{
			LogError("WSAResetEvent(SocketEvents[0x%x]) == FALSE, error code: %d", Index - WSA_WAIT_EVENT_0, WSAGetLastError());
		}
		
		//Extra sleep?
		if(++ExtraSleepCounter >= ExtraSleepValue)
		{
			SleepEx(5, 0);
			ExtraSleepCounter = 0;
		}
		
		return(1);
	}
	
	return(0);
}

//NetSleep stops sleeping if new packet is received
void WINAPI NetSleep(const DWORD dwSleep)
{
	if(!SocketEvents)
	{
		//Old style if no sockets detected
		SleepEx(dwSleep, 0);
		
		return;
	}
	
	//mode selection
	switch((int)cvar_booster_lite_mode->value)
	{
	//
	// Mode 0 - Default
	//	
	case 0:
	{
		//Simple NetSleep
		SimpleNetSleep(GetExtraSleepValue(), dwSleep);
	}
	break;
	
	//
	// Mode 1 - Sleep more if no packets received
	//
	case 1:
	{
		//Get (and fix) extra sleep frequency
		int ExtraSleepValue = GetExtraSleepValue();
		
		//If sleeping was stopped by socket event return .. otherwise sleep more.
		if(SimpleNetSleep(ExtraSleepValue, dwSleep))
			return;
		
		//Simple NetSleep
		SimpleNetSleep(ExtraSleepValue, dwSleep);
	}
	break;
	
	//
	// Mode 2 - Extended sleep interval
	//
	case 2:
	{	
		//Simple NetSleep with extended sleep interval
		SimpleNetSleep(GetExtraSleepValue(), 50);
	}
	break;
	
	// Mode 3 - Booster disabled
	case 3:
	{
		SleepEx(dwSleep, 0);
	}
	break;
	
	//
	// Invalid mode
	//
	default:
	{
		LogError("Invalid sleeping mode selection detected [%i]. Setting back to default [0].", (int)cvar_booster_lite_mode->value);
		
		CVAR_SET_STRING("booster_lite_mode", "0");
		
		//...
		SleepEx(100, 0);
	}
	break;
	
	}//switch
}

//recvfrom hook for getting socket
int PASCAL recvfrom_hook(SOCKET s, char FAR* buf, int len, int flags, struct sockaddr FAR* from, int FAR* fromlen)
{
	//check if we know this socket already
	for(unsigned long i = 0; i < list_next; i++)
		if(Sockets[i] == s)
			return(recvfrom(s,buf,len,flags,from,fromlen));
	
	//check list size
	while(list_next >= list_max)
	{
		int old_max = list_max;
		list_max += 4;
		
		Sockets = (SOCKET *)realloc(Sockets, list_max * sizeof(Sockets[0]));
		SocketEvents = (WSAEVENT *)realloc(SocketEvents, list_max * sizeof(SocketEvents[0]));
		
		memset(&Sockets[old_max], 0, sizeof(Sockets[0]) * 4);
		memset(&SocketEvents[old_max], 0, sizeof(SocketEvents[0]) * 4);
	}
	
	Sockets[list_next] = s;
	SocketEvents[list_next] = WSACreateEvent();
	
	if(SocketEvents[list_next] == 0 || (int)SocketEvents[list_next] == -1)
		LogError("WSACreateEvent() == 0x%x, error code: %d", SocketEvents[list_next], WSAGetLastError());
	
	if(WSAEventSelect(s, SocketEvents[list_next], FD_READ) == SOCKET_ERROR)
		LogError("WSAEventSelect(0x%x,0x%x,FD_READ) == SOCKET_ERROR, error code: %d", s, SocketEvents[list_next], WSAGetLastError());
	
	list_next++;
	
	return(recvfrom(s,buf,len,flags,from,fromlen));
}
